@@ -1,9 +1,7 @@
 package Mouse::Meta::Attribute;
 use strict;
 use warnings;
-require overload;
 
-use Carp 'confess';
 use Scalar::Util ();
 use Mouse::Meta::TypeConstraint;
 use Mouse::Meta::Method::Accessor;
@@ -16,21 +14,35 @@ sub new {
     $options{init_arg} = $name
         unless exists $options{init_arg};
 
-    $options{is} ||= '';
+    my $is = $options{is} ||= '';
+
+    if($is eq 'rw'){
+        $options{accessor} = $name if !exists $options{accessor};
+    }
+    elsif($is eq 'ro'){
+        $options{reader}   = $name if !exists $options{reader};
+    }
 
     bless \%options, $class;
 }
 
+# readers
+
 sub name                 { $_[0]->{name}                   }
 sub associated_class     { $_[0]->{associated_class}       }
+
+sub accessor             { $_[0]->{accessor}               }
+sub reader               { $_[0]->{reader}                 }
+sub writer               { $_[0]->{writer}                 }
+sub predicate            { $_[0]->{predicate}              }
+sub clearer              { $_[0]->{clearer}                }
+sub handles              { $_[0]->{handles}                }
+
 sub _is_metadata         { $_[0]->{is}                     }
 sub is_required          { $_[0]->{required}               }
 sub default              { $_[0]->{default}                }
 sub is_lazy              { $_[0]->{lazy}                   }
 sub is_lazy_build        { $_[0]->{lazy_build}             }
-sub predicate            { $_[0]->{predicate}              }
-sub clearer              { $_[0]->{clearer}                }
-sub handles              { $_[0]->{handles}                }
 sub is_weak_ref          { $_[0]->{weak_ref}               }
 sub init_arg             { $_[0]->{init_arg}               }
 sub type_constraint      { $_[0]->{type_constraint}        }
@@ -43,10 +55,16 @@ sub builder              { $_[0]->{builder}                }
 sub should_auto_deref    { $_[0]->{auto_deref}             }
 sub should_coerce        { $_[0]->{should_coerce}          }
 
-sub has_default          { exists $_[0]->{default}         }
+# predicates
+
+sub has_accessor         { exists $_[0]->{accessor}        }
+sub has_reader           { exists $_[0]->{reader}          }
+sub has_writer           { exists $_[0]->{writer}          }
 sub has_predicate        { exists $_[0]->{predicate}       }
 sub has_clearer          { exists $_[0]->{clearer}         }
 sub has_handles          { exists $_[0]->{handles}         }
+
+sub has_default          { exists $_[0]->{default}         }
 sub has_type_constraint  { exists $_[0]->{type_constraint} }
 sub has_trigger          { exists $_[0]->{trigger}         }
 sub has_builder          { exists $_[0]->{builder}         }
@@ -56,61 +74,18 @@ sub _create_args {
     $_[0]->{_create_args}
 }
 
-sub inlined_name {
+sub accessor_metaclass { 'Mouse::Meta::Method::Accessor' }
+
+sub _inlined_name {
     my $self = shift;
-    my $name = $self->name;
-    my $key   = "'" . $name . "'";
-    return $key;
+    return sprintf '"%s"', quotemeta $self->name;
 }
 
-sub generate_predicate {
-    my $attribute = shift;
-    my $key = $attribute->inlined_name;
-
-    my $predicate = 'sub { exists($_[0]->{'.$key.'}) }';
-
-    my $sub = eval $predicate;
-    confess $@ if $@;
-    return $sub;
-}
-
-sub generate_clearer {
-    my $attribute = shift;
-    my $key = $attribute->inlined_name;
-
-    my $clearer = 'sub { delete($_[0]->{'.$key.'}) }';
-
-    my $sub = eval $clearer;
-    confess $@ if $@;
-    return $sub;
-}
-
-sub generate_handles {
-    my $attribute = shift;
-    my $reader = $attribute->name;
-    my %handles = $attribute->_canonicalize_handles($attribute->handles);
-
-    my %method_map;
-
-    for my $local_method (keys %handles) {
-        my $remote_method = $handles{$local_method};
-
-        my $method = 'sub {
-            my $self = shift;
-            $self->'.$reader.'->'.$remote_method.'(@_)
-        }';
-
-        $method_map{$local_method} = eval $method;
-        confess $@ if $@;
-    }
-
-    return \%method_map;
-}
 
 sub create {
     my ($self, $class, $name, %args) = @_;
 
-    $args{name} = $name;
+    $args{name}             = $name;
     $args{associated_class} = $class;
 
     %args = $self->canonicalize_args($name, %args);
@@ -120,13 +95,6 @@ sub create {
         if exists $args{coerce};
 
     if (exists $args{isa}) {
-        confess "Got isa => $args{isa}, but Mouse does not yet support parameterized types for containers other than ArrayRef and HashRef (rt.cpan.org #39795)"
-            if $args{isa} =~ /^([^\[]+)\[.+\]$/ &&
-               $1 ne 'ArrayRef' &&
-               $1 ne 'HashRef'  &&
-               $1 ne 'Maybe'
-        ;
-
         my $type_constraint = delete $args{isa};
         $args{type_constraint}= Mouse::Util::TypeConstraints::find_or_create_isa_type_constraint($type_constraint);
     }
@@ -137,28 +105,20 @@ sub create {
 
     $class->add_attribute($attribute);
 
-    # install an accessor
-    if ($attribute->_is_metadata eq 'rw' || $attribute->_is_metadata eq 'ro') {
-        my $code = Mouse::Meta::Method::Accessor->generate_accessor_method_inline(
-            $attribute,
-        );
-        $class->add_method($name => $code);
-    }
+    my $associated_methods = 0;
 
-    for my $method (qw/predicate clearer/) {
-        my $predicate = "has_$method";
-        if ($attribute->$predicate) {
-            my $generator = "generate_$method";
-            my $coderef = $attribute->$generator;
-            $class->add_method($attribute->$method => $coderef);
+    my $generator_class = $self->accessor_metaclass;
+    foreach my $type(qw(accessor reader writer predicate clearer handles)){
+        if(exists $attribute->{$type}){
+            my $installer    = '_install_' . $type;
+            $generator_class->$installer($attribute, $attribute->{$type}, $class);
+            $associated_methods++;
         }
     }
 
-    if ($attribute->has_handles) {
-        my $method_map = $attribute->generate_handles;
-        for my $method_name (keys %$method_map) {
-            $class->add_method($method_name => $method_map->{$method_name});
-        }
+    if($associated_methods == 0 && ($attribute->_is_metadata || '') ne 'bare'){
+        Carp::cluck(qq{Attribute ($name) of class }.$class->name.qq{ has no associated methods (did you mean to provide an "is" argument?)});
+
     }
 
     return $attribute;
@@ -192,31 +152,31 @@ sub validate_args {
     my $name = shift;
     my $args = shift;
 
-    confess "You can not use lazy_build and default for the same attribute ($name)"
+    $self->throw_error("You can not use lazy_build and default for the same attribute ($name)")
         if $args->{lazy_build} && exists $args->{default};
 
-    confess "You cannot have lazy attribute ($name) without specifying a default value for it"
+    $self->throw_error("You cannot have lazy attribute ($name) without specifying a default value for it")
         if $args->{lazy}
         && !exists($args->{default})
         && !exists($args->{builder});
 
-    confess "References are not allowed as default values, you must wrap the default of '$name' in a CODE reference (ex: sub { [] } and not [])"
+    $self->throw_error("References are not allowed as default values, you must wrap the default of '$name' in a CODE reference (ex: sub { [] } and not [])")
         if ref($args->{default})
         && ref($args->{default}) ne 'CODE';
 
-    confess "You cannot auto-dereference without specifying a type constraint on attribute ($name)"
+    $self->throw_error("You cannot auto-dereference without specifying a type constraint on attribute ($name)")
         if $args->{auto_deref} && !exists($args->{isa});
 
-    confess "You cannot auto-dereference anything other than a ArrayRef or HashRef on attribute ($name)"
+    $self->throw_error("You cannot auto-dereference anything other than a ArrayRef or HashRef on attribute ($name)")
         if $args->{auto_deref}
         && $args->{isa} !~ /^(?:ArrayRef|HashRef)(?:\[.*\])?$/;
 
     if ($args->{trigger}) {
         if (ref($args->{trigger}) eq 'HASH') {
-            Carp::carp "HASH-based form of trigger has been removed. Only the coderef form of triggers are now supported.";
+            $self->throw_error("HASH-based form of trigger has been removed. Only the coderef form of triggers are now supported.");
         }
 
-        confess "Trigger must be a CODE ref on attribute ($name)"
+        $self->throw_error("Trigger must be a CODE ref on attribute ($name)")
             if ref($args->{trigger}) ne 'CODE';
     }
 
@@ -236,7 +196,7 @@ sub verify_against_type_constraint {
 
 sub verify_type_constraint_error {
     my($self, $name, $value, $type) = @_;
-    Carp::confess("Attribute ($name) does not pass the type constraint because: " . $type->get_message($value));
+    $self->throw_error("Attribute ($name) does not pass the type constraint because: " . $type->get_message($value));
 }
 
 sub coerce_constraint { ## my($self, $value) = @_;
@@ -256,7 +216,7 @@ sub _canonicalize_handles {
         return map { $_ => $_ } @$handles;
     }
     else {
-        confess "Unable to canonicalize the 'handles' option with $handles";
+        $self->throw_error("Unable to canonicalize the 'handles' option with $handles");
     }
 }
 
@@ -280,7 +240,14 @@ sub get_parent_args {
         return %{ $super_attr->_create_args };
     }
 
-    confess "Could not find an attribute by the name of '$name' to inherit from";
+    $self->throw_error("Could not find an attribute by the name of '$name' to inherit from");
+}
+
+sub throw_error{
+    my $self = shift;
+
+    my $metaclass = (ref $self && $self->associated_class) || 'Mouse::Meta::Class';
+    $metaclass->throw_error(@_, depth => 1);
 }
 
 1;
@@ -348,22 +315,6 @@ installed. Some error checking is done.
 
 Informational methods.
 
-=head2 generate_accessor -> CODE
-
-Creates a new code reference for the attribute's accessor.
-
-=head2 generate_predicate -> CODE
-
-Creates a new code reference for the attribute's predicate.
-
-=head2 generate_clearer -> CODE
-
-Creates a new code reference for the attribute's clearer.
-
-=head2 generate_handles -> { MethodName => CODE }
-
-Creates a new code reference for each of the attribute's handles methods.
-
 =head2 verify_against_type_constraint Item -> 1 | ERROR
 
 Checks that the given value passes this attribute's type constraint. Returns 1
